"""
Simple Accessibility Tool in Python
Sean B. Palmer, http://purl.org/net/sbp/

This can check alt attributes in a page, and check for XML 
well-formedness. It outputs EARL, using the earlapi module.

example:-
>>> import accesstool
>>> print accesstool.doAltTest('http://aaronsw.com/')
>>> print accesstool.doXMLTest('http://aaronsw.com/')

You may have to fix Expat (search for "elif a_uri", and subst):-
[[[
            elif a_uri:
                try: prefix = self._current_context[a_uri]
                except KeyError: prefix = None
]]] - /lib/xml/dom/pulldom.py
"""

import eep, earlapi
import xml.dom.minidom
from HTMLParser import HTMLParser
from urllib2 import urlopen

# Parse as XML; check alt attributes

class AltChecker(HTMLParser):
   def __init__(self, input=None): 
      HTMLParser.__init__(self)
      self.results = []
      if input: self.feed(input)

   def handle_starttag(self, tag, attrs): 
      if tag.lower() == 'img': 
         alt = 0
         for attr in attrs: 
            if attr[0].lower() == 'alt': alt = attr
         if not alt: self.results.append(0)
         else: 
            if alt[1].strip() == '': self.results.append(0)
            else: self.results.append(1)

   def handle_endtag(self, tag): 
      if tag.lower() == 'body': 
         if (0 in self.results) and (1 in self.results): 
            self.result = 'fails'
         elif (0 in self.results) and not (1 in self.results): 
            self.result = 'fails'
         elif not (0 in self.results) and (1 in self.results): 
            self.result = 'passes'
         else: self.result = 'notApplicableTo'

def checkXML(s): 
   try: 
      xml.dom.minidom.parseString(s)
      return 'passes'
   except: return 'fails'

# Store results as EARL

ToolName, ToolURI = 'accesstool.py', 'http://infomesh.net/2002/access/#Tool'
earl = earlapi.earl
rdf = earlapi.rdf
rdfs = earlapi.rdfs

# Summary:- 
# makeAssertor(URI, type, name, email)
# makeTestSubject(URI, type, thing)
# makeTestCase(URI, label, TestID, comment)
# makeAssertion(URI, TestSubject(), validity, TestCase())
# makeEvaluation(Assertor(), Assertion())

Assertor = earlapi.makeAssertor(URI=ToolURI, type=earl.Automaton, name=ToolName)
AltTC = earlapi.makeTestCase(label='Decent-Altattrs. TestCase', 
   TestID='http://infomesh.net/2002/access/#AltTest', 
   comment='Test for all alt attrs. in document: no/empty ones => fail')
ValidTC = earlapi.makeTestCase(label='Well-Formedness TestCase', 
   TestID='http://infomesh.net/2002/access/#ValidTest', 
   comment='Test for XML well-formedness')

def doAltTest(uri): 
   """Tests a resource for decent alt attrs., and returns an EARL report."""
   Validity = AltChecker(urlopen(uri).read()).result
   TestSubject = earlapi.makeTestSubject(type='WebContent', thing=uri)
   Assertion = earlapi.makeAssertion(TestSubject=TestSubject, validity=Validity, 
                          TestCase=AltTC)
   return earlapi.makeEvaluation(Assertor, Assertion)

def doXMLTest(uri): 
   """Tests a resource for XML well-formedness, and returns an EARL report."""
   Validity = checkXML(urlopen(uri).read())
   TestSubject = earlapi.makeTestSubject(type='WebContent', thing=uri)
   Assertion = earlapi.makeAssertion(TestSubject=TestSubject, validity=Validity, 
                          TestCase=ValidTC)
   return earlapi.makeEvaluation(Assertor, Assertion)

def test(): 
   import earlapi as e
   print e.EARLQuickParser(doXMLTest('http://www.w3.org/')).evaluationDetails()
   print e.EARLQuickParser(doAltTest('http://www.w3.org/')).evaluationDetails()

def bigTest(): 
   import earlapi as earl
   w3xml = doXMLTest('http://www.w3.org/') # Test W3C homepage for XML w-f
   w3alt = doAltTest('http://www.w3.org/') # Test the W3C homepage for alt tags
   aalt = doAltTest('http://aaronsw.com/') # Test Aaron's homepage for alt tags

   # .evaluationDetails() converts EARL => Human readable summary
   print earl.EARLQuickParser(w3xml).evaluationDetails() # Summary for W3C-XML
   print earl.EARLQuickParser(aalt).evaluationDetails() # Summary for Aaron-Alt
   print
   print w3xml # print the results for W3C-XML in EARL/NTriples
   print w3alt # print the results for W3C-Alt in EARL/NTriples
   print eep.nt2xmlrdf(w3alt) # Print W3C-Alt in XML/RDF

if __name__=="__main__": 
   test()