"""An EARL API, by Sean B. Palmer

This can:-

   * Parse canonicalized EARL
   * Add framework extensions
   * Store as an object
   * Query the objects
   * Summarize results

It should also be runnable to some extent as a CGI.
"""

import sys, re
try: import xml.parsers.expat
except: pass
import eep # local

# First, let's define a canonical form for EARL, and an XML parser sub class

RDF_NS = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
RDFS_NS = 'http://www.w3.org/2000/01/rdf-schema#'
EARL_NS = 'http://www.w3.org/2001/03/earl/1.0#' # let's use the real NS

rdf = eep.Namespace(RDF_NS) # the RDF namespace
rdfs = eep.Namespace(RDFS_NS) # the RDFS namespace
earl = eep.Namespace(EARL_NS) # the EARL namespace

# Simple Expat test

class ExpatTestParser: 
   def __init__(self, parser): 
      parser.StartElementHandler = self.start_element
      parser.EndElementHandler = self.end_element
   def start_element(self, name, attrs):
      print 'Start element:', name, attrs
   def end_element(self, name):
      print 'End element:', name

def expatTest(): 
   earl = """<rdf:RDF 
          xmlns="http://www.w3.org/2001/03/earl/1.0-test#"
          xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
      </rdf:RDF>"""
   p = xml.parsers.expat.ParserCreate(namespace_separator=' ')
   ExpatTestParser(p)
   p.Parse(earl)

# Right, here we go

class Statement: 
   __vdict = None
   __vdict_name = locals().keys()[0]
   def __init__(self): 
      self.__dict__[self.__vdict_name] = {}
   def __geitem__(self, name): return self.__vdict[name]
   def __setitem__(self, name, value): self.__vdict[name] = value
   def __repr__(self): return str(self.__vdict)
   def keys(self): return self.__vdict.keys()

class Assertor(Statement): 
   """This is an Assertor in EARL - like a dictionary."""
   type = earl.Assertor

class TestSubject(Statement): 
   type = earl.TestSubject

class TestCase(Statement): 
   type = earl.TestCase

class Evaluation: 
   """This is an Evaluation in EARL"""
   def __init__(self): 
      self.type = earl.Evaluation
      self.uri = '' # The URI for the Evaluation
      self.assertor = '' # The assertor for the Evaluation: an Assertor instance

# # # # # # # # # # # # # # # # # # # #
# 
# Some creative stuff: EARL Building Blocks
# 
# The following makeX functions return bits of EARL depending upon what 
# you feed into them. You can then store them, and link them together - 
# think of them as EARL building blocks.

def makeAssertor(URI=None, type=None, name=None, email=None): 
   Assertor = ''
   if URI: ID = '<'+URI+'>'
   else: ID = eep.b()
   Assertor += '%s <%s> <%s> .\n' % (ID, rdf.type, earl.Assertor)
   if type: Assertor += '%s <%s> <%s> .\n' % (ID, rdf.type, type)
   if name: Assertor += '%s <%s> "%s" .\n' % (ID, earl.name, name)
   if email: Assertor += '%s <%s> <mailto:%s> .\n' % (ID, earl.email, email)
   return [ID, Assertor]

def makeTestSubject(URI=None, type=None, thing=None): 
   TestSubject = ''
   if URI: ID = '<'+URI+'>'
   else: ID = eep.b()
   TestSubject += '%s <%s> <%s> .\n' % (ID, rdf.type, EARL_NS+type)
   tn = earlDateTime()
   TestSubject += '%s <%s> "%s" .\n' % (ID, earl.date, tn)
   if type: TestSubject += '%s <%s> <%s> .\n' % (ID, rdf.type, EARL_NS+type)
   if thing: 
      if type == 'WebContent': 
         TestSubject += '%s <%s> <%s> .\n' % (ID, earl.reprOf, thing)
   return [ID, TestSubject]

def makeTestCase(URI=None, label=None, TestID=None, comment=None): 
   """Creates and serializes an EARL test case.
      Returns an object that is a list with two components: the 
      EARL test case serialized as NTriples, and the ID for the 
      test as a RUID bNode/URI."""
   TestCase = ''
   if URI: ID = '<'+URI+'>'
   else: ID = eep.b()
   TestCase += '%s <%s> <%s> .\n' % (ID, rdf.type, earl.TestCase)
   if label: TestCase += '%s <%s> "%s" .\n' % (ID, rdfs.label, label)
   if id: TestCase += '%s <%s> "%s" .\n' % (ID, earl.id, TestID)
   if comment: TestCase += '%s <%s> "%s" .\n' % (ID, rdfs.comment, comment)
   return [ID, TestCase]

def makeAssertion(URI=None, TestSubject=None, validity=None, TestCase=None): 
   Assertion = ''
   if TestSubject[0] == TestCase[0]: raise 'ID problem, hooray! '+TestCase[0]
   if URI: ID = URI
   else: ID = eep.b()
   Assertion += '%s <%s> %s .\n' % (ID, rdf.subject, TestSubject[0])
   Assertion += '%s <%s> <%s> .\n' % (ID, rdf.predicate, EARL_NS+validity)
   Assertion += '%s <%s> %s .\n' % (ID, rdf.object, TestCase[0])
   Assertion += TestSubject[1]+TestCase[1]
   return [ID, Assertion]

def makeEvaluation(Assertor, Assertion): 
   Evaluation = ''
   if Assertor[0] == Assertion[0]: raise 'ID problem, hooray! '+Assertor[0]
   Evaluation += '%s <%s> %s .\n' % (Assertor[0], earl.asserts, Assertion[0])
   Evaluation += Assertor[1]+Assertion[1]
   return Evaluation

# # # # # # # # # # # # # # # # # # # # #
# 
# Statistics and Reports
# 
# Now for some data summarizing stuff. This is just 
# test-grade at the moment - called from EARLQuickParser

def evaluationReport(evaluation, store): 
   """Evaluation report. Only works if there's one evaluation in the store."""
   AssertionID, r = evaluation[2], '' # an Article
   # TestSubject query
   Assertion = eep.nquery('%s ?x ?y .' % AssertionID, store)
   # Get the validity
   Validity = eep.nquery('?x <%s> ?y .' % rdf.predicate, Assertion)[0][2] # P
   if '#' in Validity.v: Validity = Validity.v.split('#')[1] # Val. a Article!
   TestSubjectID = eep.nquery('?x <%s> ?y .' % rdf.subject, Assertion)[0][2]
   TestCaseID = eep.nquery('?x <%s> ?y .' % rdf.object, Assertion)[0][2]
   TestSubj = eep.nquery('%s ?x ?y .' % TestSubjectID, store)
   TestCase = eep.nquery('%s ?x ?y .' % TestCaseID, store)
   # try/except is a bit of a hack: should use result lengths instead
   # get ReprOf if there is one
   try: r += eep.nquery('?x <%s> ?y .' % earl.reprOf, TestSubj)[0][2].r+' '
   except: r += ''
   # get date if there is one
   try: r += 'of '+str(eep.nquery('?x <%s> ?y .' % earl.date, TestSubj)[0][2].v)
   except: r += ' '
   r += ' '+Validity # passes/fails etc.
   try: r += ' '+eep.nquery('?x <%s> ?y .' % rdfs.label, TestCase)[0][2].r
   except: r += ''
   try: r += ' ('+eep.nquery('?x <%s> ?y .' % rdfs.comment, TestCase)[0][2].v+')'
   except: r += ''
   return r

# # # # # # # # # # # # # # # # # # # # # # #
# 
# EARL Parsers
# 
# A couple of EARL 1.0 parsers:- 
# 
# EARLQuickParser is a simple little grubby Eep RDF API 
# store parser. It's quite effective, and does most of the 
# work in the examples.
# 
# EARLExpatParser is a test-grade canonical XML RDF EARL 1.0 
# parser. It doesn't really work at the moment.

class EARLQuickParser: 
   """A quick EARL Parser."""
   def __init__(self, s=None): 
      # You can only feed it NTriples
      self.x = eep.Article('?x')
      if s: self.store = eep.parse(s)
      else: self.store = []
      self.sort()

   def feedNTriples(self, ntriples): 
      for triple in eep.parse(ntriples): self.store.append(triple)

   def feed(self, triples): 
      for triple in triples: self.store.append(triple)

   def sort(self): 
      self.dumpEvaluations()

   def dumpEvaluations(self): 
      q = eep.parse('?x <%s> ?y .' % earl.asserts)[0]
      self.evaluations = eep.query(q, self.store)

   def assertorDetails(self): 
      result = ''
      for eval in self.evaluations: 
         result += '\n'+eval[0].r+': \n'
         r = eep.nquery('<%s> ?x ?y .' % eval[0].r, self.store)
         for x in r: 
            if (x[1].v != earl.asserts) and (x[1].v[:len(EARL_NS)] == EARL_NS): 
                   result += '   '+x[1].v+': '+x[2].v+'\n'
      return result

   def evaluationDetails(self): 
      r = ''
      for eval in self.evaluations: r += evaluationReport(eval, self.store[:])
      return r

   def countEvaluations(self): 
      return len(self.evaluations)

# Now for the Expat XML parser
# If Expat doesn't work on your system, don't 
# be surprised, but do scream. It doesn't work 
# for me in CygWin

class EARLExpatParser: 
   """A parser for canonical EARL 1.0, built on Expat.

      p = xml.parsers.expat.ParserCreate(namespace_separator=' ')
      EARLExpatParser(p)
      p.Parse(earl)"""

   def __init__(self, parser): 
      parser.StartElementHandler = self.startElement
      parser.EndElementHandler = self.endElement
      parser.CharacterDataHandler = self.charData
      self.element = ''
      self.state = ''
      self.assertor = ''
      self.evaluation = ''
      self.evaluations = []

   def startElement(self, name, attributes): 
      # print "evaluation = "+str(self.evaluation)
      self.element = name
      if (name == earl.Assertor) and (self.state == ''): 
         self.state = 'Assertor'
         self.assertor = Assertor()
      elif (name != earl.asserts) and (self.state == 'Assertor'): 
         if rdf.resource in attributes.keys(): 
            self.assertor[name] = attributes[rdf.resource]
      elif (name == earl.asserts) and (self.state == 'Assertor'): 
         self.state = 'asserts'
         Eval = Evaluation() # Create a new evaluation
         Eval.assertor = self.assertor
         if rdf.about in attributes.keys(): 
            Eval.uri = attributes[rdf.about]
            self.evaluation = Eval
         else: 
            self.evaluation = Eval
      elif (name == rdf.subject) and (self.state == 'asserts'): 
         self.state = 'asserts.subject'

   def endElement(self, name): # pass
      if name == rdf.RDF: 
         print """self.evaluation.uri = %s
self.evaluation.type = %s
self.assertor.keys() = %s""" % (self.evaluation.uri, self.evaluation.type, \
                                     self.assertor)

   def charData(self, data): 
      data = data.strip()
      # print self.element, ':', '"'+data+'"', self.state
      if data == '': data = None
      if ((self.element != earl.assertor) and 
         (self.state == 'Assertor') and data): 
         self.assertor[self.element] = data

# @@ Canonical EARL
# 
# Assertor+
#    earl junk
#    other junk
#    asserts+
# rdf:Description*

def earlDateTime(): 
   import time
   timenow = time.strftime('%Y-%m-%dT%H:%M:%SZ',  time.gmtime(time.time()))
   return timenow

def parseDateTime(date): 
   """Parses an EARL date. cf. http://www.w3.org/TR/xmlschema-2/#dateTime"""
   reg = re.compile(r'(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z')
   x = reg.findall(date)[0]
   # return year, month, day, hour, minute, second
   return x[0], x[1], x[2], x[3], x[4], x[5]

def EARLTests(): 
   # Test the Expat parser
   earl = """
<rdf:RDF xmlns="http://www.w3.org/2001/03/earl/1.0#"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" 
    xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#">
<Assertor>
 <email rdf:resource="mailto:bob@example.org"/>
 <name>Bob B. Bobbington</name>
 <asserts rdf:parseType="Resource">
   <rdf:subject>
    <WebContent>
      <reprOf rdf:resource="http://example.org/net/bob/"/>
      <date>2001-05-07</date>
    </WebContent>
   </rdf:subject>
   <rdf:predicate rdf:resource="http://www.w3.org/2001/03/earl/1.0#passes"/>
   <rdf:object>
    <TestCase>
       <rdfs:label>WCAG Single A Compliance Test</rdfs:label>
       <suite rdf:resource="http://www.w3.org/TR/WCAG10/"/>
       <level rdf:resource="http://www.w3.org/TR/WCAG10/#wc-priority-1"/>
       <rdfs:comment>All Priority 1 checkpoints => WCAG Single-A compliance</rdfs:comment>
    </TestCase>
   </rdf:object>
   <testMode rdf:resource="http://www.w3.org/2001/03/earl/1.0#Manual"/>
 </asserts>
</Assertor>
</rdf:RDF>
"""
   p = xml.parsers.expat.ParserCreate(namespace_separator='')
   EARLExpatParser(p)
   p.Parse(earl)

   # Test the EARLQuickParser
   earlnt = """
_:j15842 <http://www.w3.org/2001/03/earl/1.0#email> <mailto:bob@example.org> .
_:j15842 <http://www.w3.org/2001/03/earl/1.0#name> "Bob B. Bobbington" .
_:j15845 <http://www.w3.org/2001/03/earl/1.0#reprOf> <http://example.org/net/bob/> .
_:j15845 <http://www.w3.org/2001/03/earl/1.0#date> "2001-05-07" .
_:j15845 <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2001/03/earl/1.0#WebContent> .
_:j15844 <http://www.w3.org/1999/02/22-rdf-syntax-ns#subject> _:j15845 .
_:j15844 <http://www.w3.org/1999/02/22-rdf-syntax-ns#predicate> <http://www.w3.org/2001/03/earl/1.0#passes> .
_:j15848 <http://www.w3.org/2000/01/rdf-schema#label> "WCAG Single A Compliance Test" .
_:j15848 <http://www.w3.org/2001/03/earl/1.0#suite> <http://www.w3.org/TR/WCAG10/> .
_:j15848 <http://www.w3.org/2001/03/earl/1.0#level> <http://www.w3.org/TR/WCAG10/#wc-priority-1> .
_:j15848 <http://www.w3.org/2000/01/rdf-schema#comment> "All Priority 1 checkpoints => WCAG Single-A compliance" .
_:j15848 <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2001/03/earl/1.0#TestCase> .
_:j15844 <http://www.w3.org/1999/02/22-rdf-syntax-ns#object> _:j15848 .
_:j15844 <http://www.w3.org/2001/03/earl/1.0#testMode> <http://www.w3.org/2001/03/earl/1.0#Manual> .
_:j15842 <http://www.w3.org/2001/03/earl/1.0#asserts> _:j15844 .
_:j15842 <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2001/03/earl/1.0#Assertor> .
"""
   p = EARLQuickParser(earlnt)
   print p.countEvaluations()
   print p.assertorDetails()
   print p.evaluationDetails()

if __name__=="__main__": 
   EARLTests()